# various completions for pld-ftp-admin tools
# Author: Elan Ruusamäe <glen@pld-linux.org>

# return list of pld ftp trees
_pfa_tree() {
    local cur="$1"
    if [ -z "$PFA_TREES" ]; then
        local d
        local -a trees
        for d in  ~/ftp/*/; do
            d=${d%/}
            d=${d##*/}
            PFA_TREES+=" $d"
        done
        # kill leading space
        PFA_TREES=${PFA_TREES# }
    fi

    COMPREPLY=( $( compgen -W "$PFA_TREES" -- "$cur" ) )
}

# return list of files in a tree
_pfa_tree_files() {
    local tree="$1" cur="$2" file i
    local dir=~/ftp/$tree/SRPMS/.metadata

    # generate reply from dir
    COMPREPLY=( $( compgen -f -X "*.(src.rpm.info)" -- "$dir/$cur" ) )
    # filter out dirname from results
    for (( i=0; i < ${#COMPREPLY[@]}; i++ )); do
        file=${COMPREPLY[$i]##*/}
        COMPREPLY[$i]=$file
    done
}

have pfa-genindex &&
_pfa-genindex()
{
    local cur

    COMPREPLY=()
    _get_comp_words_by_ref cur

    if [[ "$cur" == -* ]]; then
        COMPREPLY=( $( compgen -W '\
            -q --quiet --index \
            --nopoldek --noyum --norpmrepo \
            --poldek --yum --rpmrepo \
            ' \
            -- "$cur" ) )
    else
        _pfa_tree "$cur"
    fi
} &&
complete -F _pfa-genindex pfa-genindex

have pfa-lintpkg &&
_pfa-lintpkg()
{
    local cur arg

    COMPREPLY=()
    _get_comp_words_by_ref cur

    if [[ "$cur" == -* ]]; then
        COMPREPLY=( $( compgen -W '-q -s --quiet' -- "$cur" ) )
    else
        # The first argument is an tree; the rest are files in a dir
        _count_args :

        if [[ $args == 1 ]]; then
            _pfa_tree "$cur"
        else
            _get_first_arg
            _pfa_tree_files "$arg" "$cur"
        fi
    fi
} &&
complete -F _pfa-lintpkg pfa-lintpkg

have pfa-signpkg &&
_pfa-signpkg()
{
    local cur

    COMPREPLY=()
    _get_comp_words_by_ref cur

    # The first argument is an tree; the rest are files in a dir
    _count_args :

    if [[ $args == 1 ]]; then
        _pfa_tree "$cur"
    else
        _pfa_tree_files "${COMP_WORDS[1]}" "$cur"
    fi
} &&
complete -F _pfa-signpkg pfa-signpkg pfa-rmpkg

have pfa-mvpkg &&
_pfa-mvpkg()
{
    local cur

    COMPREPLY=()
    _get_comp_words_by_ref cur

    # The first two arguments are tree names; the rest are files in a dir
    _count_args :

    if [[ $args == 1 ]] || [[ $args == 2 ]]; then
        _pfa_tree "$cur"
    else
        _pfa_tree_files "${COMP_WORDS[1]}" "$cur"
    fi
} &&
complete -F _pfa-mvpkg pfa-mvpkg pfa-testmvpkg

# Local variables:
# mode: shell-script
# sh-basic-offset: 4
# sh-indent-comment: t
# indent-tabs-mode: nil
# End:
# ex: ts=4 sw=4 et filetype=sh
